@@ -1,7 +1,7 @@
 package SQL::Translator::Parser::MySQL;
 
 # -------------------------------------------------------------------
-# $Id: MySQL.pm,v 1.54 2006/06/09 13:56:58 schiffbruechige Exp $
+# $Id: MySQL.pm,v 1.58 2007/03/19 17:15:24 duality72 Exp $
 # -------------------------------------------------------------------
 # Copyright (C) 2002-4 SQLFairy Authors
 #
@@ -134,7 +134,7 @@ A subset of INSERT that we ignore:
 
 use strict;
 use vars qw[ $DEBUG $VERSION $GRAMMAR @EXPORT_OK ];
-$VERSION = sprintf "%d.%02d", q$Revision: 1.54 $ =~ /(\d+)\.(\d+)/;
+$VERSION = sprintf "%d.%02d", q$Revision: 1.58 $ =~ /(\d+)\.(\d+)/;
 $DEBUG   = 0 unless defined $DEBUG;
 
 use Data::Dumper;
@@ -149,10 +149,13 @@ $::RD_ERRORS = 1; # Make sure the parser dies when it encounters an error
 $::RD_WARN   = 1; # Enable warnings. This will warn on unused rules &c.
 $::RD_HINT   = 1; # Give out hints to help fix problems.
 
+use constant DEFAULT_PARSER_VERSION => 30000;
+
 $GRAMMAR = << 'END_OF_GRAMMAR';
 
 { 
-    my ( $database_name, %tables, $table_order, @table_comments );
+    my ( $database_name, %tables, $table_order, @table_comments, %views, $view_order, %procedures, $proc_order );
+    my $delimiter = ';';
 }
 
 #
@@ -162,7 +165,7 @@ $GRAMMAR = << 'END_OF_GRAMMAR';
 # failed. -ky
 #
 startrule : statement(s) eofile { 
-    { tables => \%tables, database_name => $database_name } 
+    { tables => \%tables, database_name => $database_name, views => \%views, procedures =>\%procedures } 
 }
 
 eofile : /^\Z/
@@ -174,20 +177,22 @@ statement : comment
     | create
     | alter
     | insert
+    | delimiter
+    | empty_statement
     | <error>
 
-use : /use/i WORD ';'
+use : /use/i WORD "$delimiter"
     {
         $database_name = $item[2];
         @table_comments = ();
     }
 
-set : /set/i /[^;]+/ ';'
+set : /set/i /[^;]+/ "$delimiter"
     { @table_comments = () }
 
-drop : /drop/i TABLE /[^;]+/ ';'
+drop : /drop/i TABLE /[^;]+/ "$delimiter"
 
-drop : /drop/i WORD(s) ';'
+drop : /drop/i WORD(s) "$delimiter"
     { @table_comments = () }
 
 string :
@@ -203,9 +208,14 @@ nonstring : /[^;\'"]+/
 
 statement_body : (string | nonstring)(s?)
 
-insert : /insert/i  statement_body ';'
+insert : /insert/i  statement_body "$delimiter"
 
-alter : ALTER TABLE table_name alter_specification(s /,/) ';'
+delimiter : /delimiter/i /[\S]+/
+	{ $delimiter = $item[2] }
+
+empty_statement : "$delimiter"
+
+alter : ALTER TABLE table_name alter_specification(s /,/) "$delimiter"
     {
         my $table_name                       = $item{'table_name'};
     die "Cannot ALTER table '$table_name'; it does not exist"
@@ -219,10 +229,10 @@ alter : ALTER TABLE table_name alter_specification(s /,/) ';'
 alter_specification : ADD foreign_key_def
     { $return = $item[2] }
 
-create : CREATE /database/i WORD ';'
+create : CREATE /database/i WORD "$delimiter"
     { @table_comments = () }
 
-create : CREATE TEMPORARY(?) TABLE opt_if_not_exists(?) table_name '(' create_definition(s /,/) /(,\s*)?\)/ table_option(s?) ';'
+create : CREATE TEMPORARY(?) TABLE opt_if_not_exists(?) table_name '(' create_definition(s /,/) /(,\s*)?\)/ table_option(s?) "$delimiter"
     { 
         my $table_name                       = $item{'table_name'};
         $tables{ $table_name }{'order'}      = ++$table_order;
@@ -275,7 +285,7 @@ create : CREATE TEMPORARY(?) TABLE opt_if_not_exists(?) table_name '(' create_de
 
 opt_if_not_exists : /if not exists/i
 
-create : CREATE UNIQUE(?) /(index|key)/i index_name /on/i table_name '(' field_name(s /,/) ')' ';'
+create : CREATE UNIQUE(?) /(index|key)/i index_name /on/i table_name '(' field_name(s /,/) ')' "$delimiter"
     {
         @table_comments = ();
         push @{ $tables{ $item{'table_name'} }{'indices'} },
@@ -287,6 +297,48 @@ create : CREATE UNIQUE(?) /(index|key)/i index_name /on/i table_name '(' field_n
         ;
     }
 
+create : CREATE /trigger/i NAME not_delimiter "$delimiter"
+	{
+		@table_comments = ();
+	}
+
+create : CREATE PROCEDURE NAME not_delimiter "$delimiter"
+	{
+		@table_comments = ();
+        my $func_name = $item[3];
+        my $owner = '';
+        my $sql = "$item[1] $item[2] $item[3] $item[4]";
+        
+        $procedures{ $func_name }{'order'}  = ++$proc_order;
+        $procedures{ $func_name }{'name'}   = $func_name;
+        $procedures{ $func_name }{'owner'}  = $owner;
+        $procedures{ $func_name }{'sql'}    = $sql;
+	}
+
+PROCEDURE : /procedure/i
+	| /function/i
+
+create : CREATE algorithm /view/i NAME not_delimiter "$delimiter"
+	{
+		@table_comments = ();
+        my $view_name = $item[4];
+        my $sql = "$item[1] $item[2] $item[3] $item[4] $item[5]";
+        
+        # Hack to strip database from function calls in SQL
+        $sql =~ s#`\w+`\.(`\w+`\()##g;
+        
+        $views{ $view_name }{'order'}  = ++$view_order;
+        $views{ $view_name }{'name'}   = $view_name;
+        $views{ $view_name }{'sql'}    = $sql;
+	}
+
+algorithm : /algorithm/i /=/ WORD
+	{
+		$return = "$item[1]=$item[3]";
+	}
+
+not_delimiter : /.*?(?=$delimiter)/is
+
 create_definition : constraint 
     | index
     | field
@@ -301,13 +353,14 @@ comment : /^\s*(?:#|-{2}).*\n/
         $return     = $comment;
     }
 
-comment : /\/\*/ /[^\*]+/ /\*\// ';'
+comment : /\/\*/ /.*?\*\//s
     {
         my $comment = $item[2];
+        $comment = substr($comment, 0, -2);
         $comment    =~ s/^\s*|\s*$//g;
         $return = $comment;
     }
-
+    
 field_comment : /^\s*(?:#|-{2}).*\n/ 
     { 
         my $comment =  $item[1];
@@ -717,6 +770,10 @@ sub parse {
         return $translator->error("Error instantiating Parse::RecDescent ".
             "instance: Bad grammer");
     }
+    
+    # Preprocess for MySQL-specific and not-before-version comments from mysqldump
+    my $parser_version = $translator->parser_args->{mysql_parser_version} || DEFAULT_PARSER_VERSION;
+    while ( $data =~ s#/\*!(\d{5})?(.*?)\*/#($1 && $1 > $parser_version ? '' : $2)#es ) {}
 
     my $result = $parser->startrule($data);
     return $translator->error( "Parse failed." ) unless defined $result;
@@ -824,6 +881,27 @@ sub parse {
             ) or die $table->error;
         }
     }
+    
+    my @procedures = sort { 
+        $result->{procedures}->{ $a }->{'order'} <=> $result->{procedures}->{ $b }->{'order'}
+    } keys %{ $result->{procedures} };
+    foreach my $proc_name (@procedures) {
+    	$schema->add_procedure(
+    		name  => $proc_name,
+    		owner => $result->{procedures}->{$proc_name}->{owner},
+    		sql   => $result->{procedures}->{$proc_name}->{sql},
+		);
+    }
+
+    my @views = sort { 
+        $result->{views}->{ $a }->{'order'} <=> $result->{views}->{ $b }->{'order'}
+    } keys %{ $result->{views} };
+    foreach my $view_name (keys %{ $result->{views} }) {
+    	$schema->add_view(
+    		name => $view_name,
+    		sql  => $result->{views}->{$view_name}->{sql},
+		);
+    }
 
     return 1;
 }
