@@ -85,12 +85,14 @@ sub as_graph_pm {
 
 =pod
 
-=head2 as_grap_pmh
+=head2 as_graph_pm
 
 Returns a Graph::Directed object with the table names for nodes.
 
 =cut
 
+    require Graph::Directed;
+
     my $self = shift;
     my $g    = Graph::Directed->new;
     
@@ -590,11 +592,8 @@ Returns all the tables as an array or array reference.
       sort { $a->[0] <=> $b->[0] }
       map  { [ $_->order, $_ ] } values %{ $self->{'tables'} };
 
-## TT DEBUG
-#print STDERR "Wantarray: ", wantarray, "\n";
     if (@tables) {
         return wantarray ? @tables : \@tables;
-#        return \@tables;
     }
     else {
         $self->error('No tables');
