@@ -1,7 +1,7 @@
 package SQL::Translator::Schema::Index;
 
 # ----------------------------------------------------------------------
-# $Id: Index.pm,v 1.14 2005/08/10 16:46:55 duality72 Exp $
+# $Id: Index.pm,v 1.17 2007/03/06 23:50:23 duality72 Exp $
 # ----------------------------------------------------------------------
 # Copyright (C) 2002-4 SQLFairy Authors
 #
@@ -53,7 +53,7 @@ use base 'SQL::Translator::Schema::Object';
 
 use vars qw($VERSION $TABLE_COUNT $VIEW_COUNT);
 
-$VERSION = sprintf "%d.%02d", q$Revision: 1.14 $ =~ /(\d+)\.(\d+)/;
+$VERSION = sprintf "%d.%02d", q$Revision: 1.17 $ =~ /(\d+)\.(\d+)/;
 
 my %VALID_INDEX_TYPE = (
     UNIQUE,    1,
@@ -249,14 +249,33 @@ Determines if this index is the same as another
     my $self = shift;
     my $other = shift;
     my $case_insensitive = shift;
+    my $ignore_index_names = shift;
     
     return 0 unless $self->SUPER::equals($other);
-#    return 0 unless $case_insensitive ? uc($self->name) eq uc($other->name) : $self->name eq $other->name;
-    return 0 unless $self->is_valid eq $other->is_valid;
+
+    unless ($ignore_index_names) {
+      unless ((!$self->name && ($other->name eq $other->fields->[0])) ||
+        (!$other->name && ($self->name eq $self->fields->[0]))) {
+        return 0 unless $case_insensitive ? uc($self->name) eq uc($other->name) : $self->name eq $other->name;
+      }
+    }
+    #return 0 unless $self->is_valid eq $other->is_valid;
     return 0 unless $self->type eq $other->type;
-    my $selfFields = join(":", $self->fields);
-    my $otherFields = join(":", $other->fields);
-    return 0 unless $case_insensitive ? uc($selfFields) eq uc($otherFields) : $selfFields eq $otherFields;
+    
+    # Check fields, regardless of order
+    my %otherFields = ();	# create a hash of the other fields
+    foreach my $otherField ($other->fields) {
+    	$otherField = uc($otherField) if $case_insensitive;
+    	$otherFields{$otherField} = 1;
+    }
+    foreach my $selfField ($self->fields) { # check for self fields in hash
+    	$selfField = uc($selfField) if $case_insensitive;
+    	return 0 unless $otherFields{$selfField};
+    	delete $otherFields{$selfField};
+    }
+    # Check all other fields were accounted for
+    return 0 unless keys %otherFields == 0;
+
     return 0 unless $self->_compare_objects(scalar $self->options, scalar $other->options);
     return 0 unless $self->_compare_objects(scalar $self->extra, scalar $other->extra);
     return 1;
