@@ -12,6 +12,7 @@ create unique index u_name on person (name);
 create table employee (
 	position varchar(50),
 	employee_id integer,
-	CONSTRAINT FK5302D47D93FE702E FOREIGN KEY (employee_id) REFERENCES person (person_id)
+	CONSTRAINT FK5302D47D93FE702E FOREIGN KEY (employee_id) REFERENCES person (person_id),
+	PRIMARY KEY  (position, employee_id)
 ) ENGINE=InnoDB;
 
